// jQuery Extension
$.fn.reverse = [].reverse;


var OFFSET_X = 40,
	OFFSET_Y = 40;

/** (snowman doodle) */
var snowman = {
    // Define some variables
    'drawing':          false,
	'trashAble':		false,
	'trashY':			700,
	'path':				[],
	'buffer':			[],
	'drag':				null,
	'counter':			0,
	'settings':			{
		background: 0,
		shape: 0,
		carvertype: 'carver',
		carver: 0,
		counter:0,
	}
};

snowman.init = function() {
	// initialize canvas
	$(".canvas").attr({width:480, height:800});
    
	// setup mask canvas
	snowman.mask = $("#mask").css('z-index', 10).get(0);
	snowman.maskcontext = snowman.mask.getContext('2d');
	
	// setup canvas
    snowman.canvas = $('#canvas').css('z-index', 15).get(0);
    snowman.context = snowman.canvas.getContext('2d');
	
	// setup surface canvas
    snowman.surface = $('#surface').css('z-index', 20).get(0);
    snowman.surfacecontext = snowman.surface.getContext('2d');
    
	// Touch screen based interface
    $(snowman.surface).bind('touchstart', snowman.drawStart);
    $(snowman.surface).bind('touchmove', snowman.draw);
    $(snowman.surface).bind('touchend', snowman.drawEnd);
	
	// load user data
	snowman.loadData();	
		
	// load user background
	snowman.loadBackground(snowman.settings.background, true);
	
};


// New snowman
snowman.newsnowman = function() {
	snowman.buffer = [];
	snowman.path = [];
    snowman.clearCanvas();
	snowman.saveData();
}


// loadBackground
snowman.loadBackground = function(index, noupdate) {
	if(backgrounds[index]) {
		$('body').css({'background': 'no-repeat center top '+backgrounds[index].bgcolor})
		
		if(backgrounds[index].bg) {
			$('body').css({'background-image': 'url(backgrounds/'+backgrounds[index].bg+')'})	
		}
		snowman.settings.background = index;		
		if(!noupdate) {snowman.saveData()}
	}
}




// loadCarver
snowman.loadCarver  = function(type, index){
	snowman.settings.carvertype = type;
	snowman.settings.carver = index;
}

// getCarver
snowman.getCarver  = function(e){
	switch(e?e:snowman.settings.carver) {
		case 1: return {size: 2, color: "#000000"}; break;		// coal
		default: return {size: 6, color: "#ffffff"}; break;		// snow
	}
}

// getFaceOrShape
snowman.getFaceOrShape  = function(type, index){
	if(type=="face") {
		return faces[index];
	} else {
		return shapes[index].img;		
	}
}	

// drawFaceShape
snowman.drawFaceShape = function(img, x, y, c) { 
	// transform x,y
	x = x - OFFSET_X;
	y = y - OFFSET_Y;

	var w = img.width/SCALE, h =img.height/SCALE,
		xa = x/SCALE - (w/2),
		ya = y/SCALE - (h/2);
	
	if(c) {
		snowman.context.drawImage(img, xa,ya,w,h);
	} else {
		snowman.clearCanvasSurface();
		snowman.surfacecontext.drawImage(img, xa,ya,w,h);
	}	
}

// loadShape
snowman.loadShape  = function(shape){
	if (snowlist[shape]) {
		snowman.buffer = [];
		snowman.settings.shape = shape;
		snowman.drawCanvas();
		snowman.saveData();
	}
}

// drawCanvas, redraws the canvas with the current  mask and snowman
snowman.drawCanvas = function(){
	snowman.clearCanvasCanvas();
	snowman.clearCanvasMask();
	var y = 60/SCALE;
	// set mask
	//var mask = snowlist[snowman.settings.shape].mask;
	//snowman.maskcontext.drawImage(mask, 5, y, mask.width/SCALE, mask.height/SCALE);
	if(snowman.settings.shape == 0 ) return;
	// set canvas
	var img = snowlist[snowman.settings.shape].img;
	snowman.context.drawImage(img, 40/SCALE, y, img.width/SCALE, img.height/SCALE);
}

// clearCanvas methods
snowman.clearCanvasCanvas = function() {
 	snowman.context.clearRect(0,0, snowman.canvas.width, snowman.canvas.height);
	snowman.canvas.width = snowman.canvas.width;	
}

snowman.clearCanvasMask = function() {
 	snowman.maskcontext.clearRect(0,0, snowman.mask.width, snowman.mask.height);
	snowman.mask.width = snowman.mask.width;	
}

snowman.clearCanvasSurface = function() {
 	snowman.surfacecontext.clearRect(0,0, snowman.surface.width, snowman.surface.height);
	snowman.surface.width = snowman.surface.width;
}

// clearCanvas
snowman.clearCanvas = function(ev) {
    // Clear canvas
   	snowman.clearCanvasSurface();
   
	// load current snowman
	snowman.drawCanvas();
}

snowman.getItemOnLocation = function(x, y) {
try {
	// transform x,y
	//	x = x - OFFSET_X;
	//  y = y - OFFSET_Y;	
	// get buffer
	buffer = snowman.buffer.slice();
	buffer.reverse();
	// checks if there is an shape on location
	$.each(buffer, function(index, path){
		if (path.type == "face" || path.type == "shape") {
			// get shape information
			var img = snowman.getFaceOrShape(path.type, path.carver);
			// create box
			var xw = (img.width / SCALE) / 2, xh = (img.height / SCALE) / 2, xa = path.data.x - xw, ya = path.data.y - xh;
			/* debug
			 snowman.context.fillStyle= "red";
			 snowman.context.fillRect(xa, ya, xw*2, xh*2);
			 snowman.context.fillStyle= "black";
			 snowman.context.fillRect(x-5, y-5, 5*2, 5*2); */
			if (x > xa && x < (xa + xw * 2) && y > ya && y < (ya + xh * 2)) {
				//alert("found");
				// delete index
				buffer.splice(index, 1);
				// reassign shape
				snowman.settings.carvertype = path.type;
				snowman.settings.carver = path.carver;
				// reverse
				buffer.reverse();
				// assign new buffer
				snowman.buffer = buffer.slice();
				// refresh buffer
				snowman.drawbuffer();
			}
		}
	});
}catch(e) {
	//alert(e);
}
}

snowman.showTrash = function(b) {
	if (b) {
		$("#toolbar").hide();
		$("#trash").show();
	} else {
		$("#toolbar").show();
		$("#trash").hide().css("background-color", "#000");
	}
}

snowman.drawStart = function(e) {
	// Initialize Event
 	var ev = e.originalEvent;
	
	// sanity check
	if (ev.changedTouches.length < 1) return;
	
	// stop event
	ev.preventDefault();
	ev.stopPropagation();
	
	// Calculate the current mouse X, Y coordinates with canvas offset
	var t = ev.changedTouches[0],
		x = t.pageX - $(snowman.canvas).offset().left,
		y = t.pageY - $(snowman.canvas).offset().top;
	
	// test position
	snowman.getItemOnLocation(x, y);
	
	// start drawing	
	snowman.drawing = true;
	if (snowman.settings.carvertype != "carver") {
		// initialize image drag
		snowman.clearCanvasSurface();
		snowman.drag = snowman.getFaceOrShape(snowman.settings.carvertype, snowman.settings.carver);
		snowman.drawFaceShape(snowman.drag, x, y);
		// show trash
		snowman.showTrash(true);		
	}
	else {
		// begin drawing
		
		var c = snowman.getCarver();
		snowman.context.lineWidth = c.size;
		snowman.context.strokeStyle = c.color;
		// Store the current x, y positions
		snowman.oldX = x;
		snowman.oldY = y;
		// reset path record buffer
		snowman.path = [{
			'x': x,
			'y': y
		}];
	}
}

snowman.draw = function(e) {
	// Initialize Event
 	var ev = e.originalEvent;
	
	// sanity check
	if (ev.changedTouches.length < 1) return;
	
	// stop event
	ev.preventDefault();
	ev.stopPropagation();
	
	// Calculate the current mouse X, Y coordinates with canvas offset
	var t = ev.changedTouches[0],
		x = t.pageX - $(snowman.canvas).offset().left,
		y = t.pageY - $(snowman.canvas).offset().top;
	
	if (DEBUG) {$("#canvaspos").html(x + ", " + y)}
	
	// check trash
		
    
    // If the mouse is down (drawning) then start drawing lines
    if(snowman.drawing) {
		if(snowman.settings.carvertype!="carver"){
			// live drag
			snowman.drawFaceShape(snowman.drag, x, y);
		} else {
        	snowman.context.beginPath();
        	snowman.context.moveTo(snowman.oldX, snowman.oldY);
        	snowman.context.lineTo(x, y);
        	snowman.context.closePath();
        	snowman.context.stroke();
			// update record buffer
			snowman.path.push({'x':x, 'y':y});
		}
		
		//$("#canvaspos").html(y + ", " + snowman.trashY);
		
		if (y > snowman.trashY) {
			snowman.trashAble = true;
			$("#trash").css("background-color", "#cc0000");
		}
		else {
			snowman.trashAble = false;
			$("#trash").css("background-color", "#000");
		}
    }
    // Store the current X, Y position
    snowman.oldX = x;
    snowman.oldY = y;
};

 
// Finished drawing (mouse up)
snowman.drawEnd = function(e) {
// Initialize Event
 	var ev = e.originalEvent;
	
	// sanity check
	if (ev.changedTouches.length < 1) return;
	
	// stop event
	ev.preventDefault();
	ev.stopPropagation();
		
	// end drawing
    snowman.drawing = false;

	// check carvertype
	if(snowman.settings.carvertype!="carver"){
		// Calculate the current mouse X, Y coordinates with canvas offset
		var t = ev.changedTouches[0],
			x = t.pageX - $(snowman.canvas).offset().left,
			y = t.pageY - $(snowman.canvas).offset().top;	
		// create carve object	
		snowman.path = {'x':x, 'y':y};
		snowman.clearCanvasSurface();
	}
	
	if (snowman.trashAble) {
		snowman.trashAble = false;
	}
	else {
		// fill recorded path
		snowman.drawpath(snowman.addbuffer(snowman.path));
	}
	// reset path
	snowman.path = [];	
	// hide trash
	snowman.showTrash(false);
	
}

// Add Buffer
snowman.addbuffer = function(path) {
	if(snowman.buffer.length>150) {
		alert("You have reached the maximum allowed elements on this snowman.");
		return false;
	}
	
	var item = {
		type: snowman.settings.carvertype,
		carver: snowman.settings.carver,
		data: path
	};
	snowman.buffer.push(item);
	snowman.saveData();
	return item;
}

// Draw Path
snowman.drawpath = function(path){
	switch(path.type) {
		// shape
		case "face":
		case "shape":
			snowman.drawFaceShape(snowman.getFaceOrShape(path.type, path.carver), path.data.x, path.data.y, true);
			break;
		
		// carver
		default:
			var c = snowman.getCarver(path.carver);
			snowman.context.fillStyle =  c.color;
			snowman.context.strokeStyle = c.color;
			snowman.context.lineWidth = c.size;
	
			oldpos = null;
			$.each(path.data, function(index, coords){
				if(oldpos!=null) {
					snowman.context.beginPath();
					snowman.context.moveTo(oldpos.x, oldpos.y);
					snowman.context.lineTo(coords.x, coords.y)
					snowman.context.closePath();
					snowman.context.stroke();
				}
				oldpos = coords;	
			});
			
			// autoconnect and fill if start and end are within tolerance
			var tolerance = 45,
				start = path.data[0], end = path.data[path.data.length-1],
				cx = end.x - start.x, cy = end.y - start.y;
			if(cx<0) {cx = cx * -1} 
			if(cy<0) {cy = cy * -1}
			
			var autofill = (path.carver < 1);
			if((cx<tolerance&&cy<tolerance)||autofill==true) {
				// fill
				snowman.context.beginPath();
				$.each(path.data, function(index, coords){
					if (index == 0) {
						snowman.context.moveTo(coords.x, coords.y);
					} else {
						snowman.context.lineTo(coords.x, coords.y);
					}
				});
				snowman.context.closePath();
				snowman.context.fill();
			}
	}
}

// DrawBuffer
snowman.drawbuffer = function(){
	snowman.clearCanvas();
	$.each(snowman.buffer, function(index, path) {snowman.drawpath(path)});
}


// Undo
snowman.undo = function() {
	// remove last element in array
	snowman.buffer.pop();
	snowman.drawbuffer();	
}


// saveData
snowman.saveData = function() {
	localStorage.setItem('snowman', JSON.stringify(snowman.buffer));
	localStorage.setItem('settings', JSON.stringify(snowman.settings));
	localStorage.setItem('counter', snowman.counter+"");		
}

// loadData
snowman.loadData = function() {
	try {
		snowman.settings = JSON.parse(localStorage.getItem('settings'));
		snowman.buffer = JSON.parse(localStorage.getItem('snowman'));
		var c = parseInt(localStorage.getItem('counter'));
		snowman.counter = isNaN(c)?0:c;
		/*
		if(snowman.settings.carvertype=='') {
			snowman.settings.carvertype = 'carver';
			snowman.settings.carver = 0;
		}*/		
		snowman.drawbuffer();	} 
	catch (e) {
		snowman.settings = {background:0, shape:0, carvertype: 'carver', carver: 0};
		snowman.newsnowman();
	}
}



// prepeage Image
snowman.prepareImage = function() {
	// hide toolbar and canvas
	$("#toolbar").hide();
	// create virtual canvas
	var c = $("<canvas></canvas>").attr({width:480, height:800}),
		v = c.get(0),
		vc = v.getContext('2d');
	// copy background into new canvas
	var bg = backgrounds[snowman.settings.background];
	vc.fillStyle = bg.bgcolor;
    vc.fillRect(0, 0, c.width, c.height);
	if(bg.bg&&bg.img){vc.drawImage(bg.img,0,0,bg.img.width/SCALE, bg.img.height/SCALE);}
	// draw mask
	vc.drawImage(snowman.mask, 0, 85, snowman.canvas.width/SCALE, snowman.canvas.height/SCALE);
	// draw snowman
	vc.drawImage(snowman.canvas, 0, 85, snowman.canvas.width/SCALE, snowman.canvas.height/SCALE);
	
	var w = $("<div></div>").attr("id", "screencapture").css({background:bg.bgcolor,position:'absolute',top:0,left:0, width:480, height:800, 'z-index':10000}).append(c);
	$("body").append(w);
}


// save
snowman.saveImage = function() {
	snowman.prepareImage();
	setTimeout(function() {
			FileUtil.CaptureImage("Mysnowmans", "snowman-"+snowman.counter+".jpg");
			snowman.counter += 1;
			snowman.saveData();
		}, 500); 	
}


//share
snowman.shareImage = function() {
	snowman.prepareImage();
	setTimeout(function() {
		    DroidGap.share("Mysnowmans", "snowman-"+snowman.counter+".jpg");
			snowman.counter += 1;
			snowman.saveData();
		}, 500);	
}

// Events

var ScreenCaptureComplete = function() {
	$("#screencapture").remove();
	$("#toolbar").show();
	alert("Your snowman was successfully saved.");
}

var ShareComplete = function() {
	$("#screencapture").remove();
	$("#toolbar").show();
}
